#!perl -T

use Test::More tests => 1;

BEGIN {
    use_ok( 'Lingua::ZH::PinyinConvert::ID' );
}

diag( "Testing Lingua::ZH::PinyinConvert::ID $Lingua::ZH::PinyinConvert::ID::VERSION, Perl $], $^X" );
